#!/usr/bin/python
from __future__ import print_function
#----------------------------------------------------------------------------------------
# NAME: AWSQuickSightInfo.py
# DESC: Processes all of the Info files into a single, tab delimited, output file.
#       The output file has a header line that lists all of the attributes. each
#       line of the output file contains the values of the attributes in an Info
#       file. This tool requires two parameters,  The first is the full filename
#       of the output file. The second is the pathname of the Info directory. The
#       Attributes file must be in the P2/../Tables directory.
#
#       Please note that this script is compatible with both Python v2.7 and v3.8
#
# Copyright: 2020, Visible Systems Corp. All rights reserved.
#----------------------------------------------------------------------------------------
#
# for the command line arguments
import sys
# for the file i/o routines
import os
# for searching through the input strings
import re

# Used to display the name of the program at the start
pgmName=os.path.basename(__file__)

# Let's validate the arguments (we expect 2 and 0 is the name of this script)
if len(sys.argv) == 3:
    try:
        outfile=open(sys.argv[1],"w")
    except IOError:
        print ("")
        print ("ERROR: cannot write to ",sys.argv[1])
        sys.exit()
    outfile.close()
    if not os.path.isdir(sys.argv[2]):
        print ("")
        print ("ERROR: ",sys.argv[2]," does not exist")
        sys.exit()
    try:
        attrfn  = sys.argv[2]+"\..\Tables\Attributes"
        infile=open(attrfn,"r")
    except IOError:
        print ("")
        print ("ERROR: cannot find the ",attrfn," file")
        sys.exit()
    infile.close()
    outfile.close()
else:
    print ("")
    print ("USAGE: "+pgmName+"  \"P1\"  \"P2\"")
    print ("")
    print ("       P1 = the name of the output file (it will be tab delimited)")
    print ("       P2 = the full pathname of the Razor Info folder (or a copy)")
    print ("")
    print ("       ex: .\\"+pgmName+"  \"C:\Razor Test Issues\DataSet.tsv\"   \"C:\Razor Test Issues\Info\"")
    print ("           or")
    print ("           ./"+pgmName+" /home/rzadmin/RazorIssues/DataSet.tsv /home/rzadmin/RazorIssues/Info")
    print ("")
    print ("       Requires two parameters. P1 is the path and name of the output file created by this tool")
    print ("       and P2 is the path and name of the Razor Info folder. If spaces exist, the parameters can")
    print ("       be enclosed in double quotes. The Attributes file must be in the P2/../Tables folder")
    print ("")
    print ("       This tool will loop through and parse each file in the Info folder (i.e. P2) creating a")
    print ("       tab delimited file (i.e. P1) with one line per issue.  Each line in the file contains")
    print ("       the data for a single issue. The first line in the file is a header and lists the name")
    print ("       of each attribute. This file can be used to create a Data Set in AWS QuickSight.")
    print ("")
    print ("       Please note that this script is compatible with both Python v2 and v3")
    sys.exit()

# Let's get started
print ("")
print ("start",pgmName)
#
# The input/output file and folder names
#
# As parameters from the command line
outfn = sys.argv[1]
infodir = sys.argv[2]

# Used during testing
#outfn = r"D:\Users\Alfredo\OneDrive - Space Coast Information Systems\Razor Development\QuickSight\DataSet-demo.tsv"
#infodir = r"D:\Users\Alfredo\OneDrive - Space Coast Information Systems\Razor Development\QuickSight\demoFiles\demoIssues\Info"
#outfn = r"D:\Users\Alfredo\OneDrive - Space Coast Information Systems\Razor Development\QuickSight\DataSet-support.tsv"
#infodir = r"D:\Users\Alfredo\OneDrive - Space Coast Information Systems\Razor Development\QuickSight\SupportIssues\Info"
#outfn = r"C:\Users\WSUSER\Desktop\DataSet-demo.tsv"
#infodir = r"C:\Users\WSUSER\eclipse-workspace\QuickSight\Datasets\demoFiles\demoIssues\Info"
#outfn = r"C:\Users\WSUSER\Desktop\DataSet-support.tsv"
#infodir = r"C:\Users\WSUSER\eclipse-workspace\QuickSight\Datasets\SupportIssues\Info"

# Setup the Attributes filename for parsing
attrfn  = infodir+"\..\Tables\Attributes"

# Setup global state, attr, and time lists. Used to check the values read to see if it
# is a state, or attribute, or a time type.
statesList = []
attrList = []
timeList = []

# Open the outfile and start writing the header line
outfile=open(outfn,"w")
outfile.write("IssueNum\t")
outfile.write("LatestVer\t")
outfile.write("LastModifiedDate\t")

# Used to keep track of how many attributes have been written to the output outfile
attrCount=3

# Loop through each line in the Attributes file for this issue group and build up the state list.
infile=open(attrfn)
for aline in infile:
    aline=aline[:-1]
    myregexp=re.compile(r'^STATE')
    if (myregexp.search(aline)):
        statesList=re.split("[\t]+",aline)
        statesList=statesList[2].replace(" ","")
        statesList=re.split(",",statesList)
        # Loop through the states and add to the header line
        for astate in statesList:
            # Add the state name to the header line
            outfile.write("ST-"+astate+"\t")
            attrCount += 1
        continue
    else:
        continue
infile.close()

# Loop through the Attributes file again this time to build up the attribute list.
infile=open(attrfn)
firstAttr=True
for aline in infile:
    # Get rid of the last character in the line
    aline=aline[:-1]
    if not aline.strip():
        # Ignore blank lines
        continue
    myregexp=re.compile(r'^#')
    if (myregexp.search(aline)):
        # Ignore comments
        continue
    myregexp=re.compile(r'^LABEL')
    if (myregexp.search(aline)):
        # Ignore labels
        continue
    myregexp=re.compile(r'^STATE')
    if (myregexp.search(aline)):
        # Ignore state lines since we've already processed them
        continue
    myregexp=re.compile(r'^TEXT_PANE')
    if (myregexp.search(aline)):
        # Ignore text pane lines
        continue
    myregexp=re.compile(r'^TIME_STAMP')
    if (myregexp.search(aline)):
        # This is a time stamp field
        myattr=re.split("[\t]+",aline)
        # Save the second value in the line since this is the name of the time stamp field.
        timeList.append (myattr[1])
        attrList.append (myattr[1])
        # Write the attribute name to the header line.
        outfile.write("\tATTR-"+myattr[1])
        attrCount += 1
        continue
    else:
        # This is a regular attribute and not a time stamp
        myattr=re.split("[\t]+",aline)
        attrList.append (myattr[1])
        if firstAttr:
            # This is the first attribute so we do not need to add a tab
            # before the attribute name to the output file
            firstAttr = False
            outfile.write("ATTR-"+myattr[1])
        else:
            # We need to add a tab to before each attribute name for all
            # but the first attribute.
            outfile.write("\tATTR-"+myattr[1])
        attrCount += 1
        continue

# Finished with the header line in the output file
outfile.write("\n")
# Finished with the Attributes file
infile.close()

# Display some info for use later in debugging Data Set import errors
#print "    attrCount = "+str(attrCount)+"\n"
#print statesList
#print attrList
#print timeList

# Loop through the Info files in the folder and process one file at a time.
for fn in os.listdir(infodir):
    # Set to 0 to keep track of how many attributes have been written to the output file
    myAttrCount=0
    # Compose the full name of the file (foldername + filename)
    ifn=os.path.join(infodir,fn)

    # Skip this file if it is of zero length
    if os.stat(ifn).st_size == 0:
        continue

    # The file is not empty so open it and begin processing one line at a time
    infile=open(ifn)

    print ("   processing ",fn)
    # Write the filename, which is the issue number, to the output file
    outfile.write("'"+fn+"'")
    myAttrCount += 1

    # We have not yet done the states or attribues for this file so set these values to True
    doStates=True
    doAttrs=True

    # Setup a local state and attribute list
    myStatesList=list(statesList)
    myAttrList=list(attrList)

    # Loop through each line in this Info file
    for aline in infile:
        # Get rid of the last character in the line
        aline=aline[:-1]
        # Split the line into fields seperated by one or more tabs
        mysplit=re.split("[\t]+",aline)

        # Setup regular expressions for the label expected at the start of each line
        myLVregexp=re.compile(r'^LATEST_VERSION')
        myLMregexp=re.compile(r'^LAST_MODIFIED')
        myTCregexp=re.compile(r'^TIME_CHECKED_IN')
        myCSregexp=re.compile(r'^CHECKSUM')
        myFIregexp=re.compile(r'^FILE_ID')
        myIBregexp=re.compile(r'^IS_A_BRANCH')
        myCOregexp=re.compile(r'^CHECKED_OUT')
        myBNregexp=re.compile(r'^BINARY')
        myNBregexp=re.compile(r'^NUM_BRANCHES')
        myNSregexp=re.compile(r'^NUM_STATES')
        myITregexp=re.compile(r'^IS_TERMINATED')
        myEKregexp=re.compile(r'^EXPAND_KEYWORDS')
        mySTregexp=re.compile(r'^STATE')
        myATregexp=re.compile(r'^ATTRIBUTE')

        # Check to see which type of line we are processing
        if (myLVregexp.search(aline)):
            # This is the latest version
            outfile.write("\t"+"'"+mysplit[1]+"'")
            myAttrCount += 1
            continue
        elif (myLMregexp.search(aline)):
            # This is the last modified date
            outfile.write("\t"+"'"+mysplit[2]+"'")
            myAttrCount += 1
            continue
        elif (myTCregexp.search(aline)) or (myCSregexp.search(aline)) or (myFIregexp.search(aline)) or (myIBregexp.search(aline)):
            # We ignore these lines
            continue
        elif (myCOregexp.search(aline)) or (myBNregexp.search(aline)) or (myNBregexp.search(aline)) or (myNSregexp.search(aline)):
            # We ignore these lines
            continue
        elif (myITregexp.search(aline)) or (myEKregexp.search(aline)):
            # We ignore these lines
            continue
        elif (mySTregexp.search(aline)):
            # This is a state
            num = 0
            # Find the state and build up the list (we will output these later)
            for astate in myStatesList:
                if (astate == mysplit[1]):
                    myStatesList[num]=mysplit[2]
                num += 1
            #outfile.write("\t"+mysplit[2])
            continue
        elif (myATregexp.search(aline)):
            # We are finished with states and now we process the attributes
            if (doStates):
                # We have not yet done the states so we need to put them out to the file
                num = 0
                for astate in myStatesList:
                    if (astate == statesList[num]):
                        # There is no value for this state in the info file
                        outfile.write("\t"+"'[EMPTY]'")
                        myAttrCount += 1
                    else:
                        # Write the value we found in the info file to the output file
                        outfile.write("\t"+"'"+astate+"'")
                        myAttrCount += 1
                    num += 1
                doStates=False

            # Find out which attribute we have and build up our lise (we will output these later)
            num = 0
            for anattr in myAttrList:
                if (anattr == mysplit[1]):
                    # We found the attribute
                    if (not mysplit[2]):
                        # The value is empty so we do not replace the name
                        # with the value and we go to the next attribute
                        continue
                    else:
                        # The value is not empty so we save it
                        myAttrList[num]=mysplit[2]

                    # Check to see if this is a time attribute and if it is
                    # then we need the 4th field (i.e. text value)
                    for tattr in timeList:
                        if (tattr == anattr):
                            myAttrList[num]=mysplit[3]
                            # We found the time attribute so we can end the loop
                            break
                        else:
                            # We have not found the time attribute so keep looking
                            continue
                num += 1
                continue
        else:
            continue

    if (doAttrs):
        # We have not yet written the attributes
        num = 0
        for anattr in myAttrList:
            if (anattr == attrList[num]):
                # The name is equal to the value so this means there is no value
                # and we put an empty value to the output file
                outfile.write("\t"+"'[EMPTY]'")
                myAttrCount += 1
            else:
                # The name is not equal to the value so we put the value in the
                # output file
                outfile.write("\t"+"'"+anattr+"'")
                myAttrCount += 1
            num += 1
        doAttrs=False

    # Debug statement... we check to see if the global and local attribute counts match
    #if myAttrCount <> attrCount:
    #    print fn+" attrCount="+str(attrCount)+" myAttrCount="+str(myAttrCount)

    # We are done with this info file so we output an end of line,
    # close it, and go to the next file
    outfile.write("\n")
    infile.close()

print ("end",pgmName)
